<?php
/* 
 * Returns true if $var is not white space only, and false otherwise.
 */
function not_white( $var )
{
    return trim($var) != "" ? true : false;
}

/* 
 * Returns an associative array of objectClasses for the specified 
 * $server_id. Each array entry's key is the name of the objectClass
 * in lower-case. 
 * The sub-entries consist of sub-arrays called 'must_attrs' and 
 * 'may_attrs', and sub-entries called 'oid', 'name' and 'description'.
 *
 * The bulk of this function came from the good code in the 
 * GPL'ed LDAP Explorer project. Thank you.
 */
function get_schema_objectclasses()
{
    $ds = conn();
   
    if( ! $ds )
        return false;
   
    // get all the objectClasses
    $result = @ldap_read($ds, 'cn=subschema', '(objectClass=*)',
                array( 'objectclasses' ), 0, 200, 0, LDAP_DEREF_ALWAYS );
    if( ! $result )
        $result = @ldap_read($ds, 'cn=schema', '(objectClass=*)',
                array( 'objectclasses' ), 0, 200, 0, LDAP_DEREF_ALWAYS );

    if( ! $result ) return false;
    if( $result ) $raw_oclasses = ldap_get_entries($ds,$result );
   
    // build the array of objectClasses
    $oclasses = array();
    for( $att=0; $att < count( $raw_oclasses[0]["objectclasses"] ); $att++ )
    {
        @$class = $raw_oclasses[0]["objectclasses"][$att];

        if(preg_match( "/[\s]+NAME[\s'\(]+([a-zA-Z0-9\-_]+)[\s'\)]/" , $class, $name)){
            $key = strtolower( trim( $name[1] ) );
            $oclass_name = trim( $name[1] );
            if( ! $key ) continue;
        }
        $oclasses[$key] = array();

        if(preg_match( "/[\s]+([\d\.]+)[\s]+NAME/", $class, $oid ))
            $oclasses[$key]['oid'] = trim( $oid[1] );
        else
            $oclasses[$key]['oid'] = '';

        if(preg_match( "/[\s]+DESC[\s]+'([a-zA-Z0-9\-_ ]+)'/", $class, $description ))
            $oclasses[$key]['description'] = trim( $description[1] );
        else
            $oclasses[$key]['description'] = '';

        if(preg_match( "/[\s]+SUP[\s]+([a-zA-Z0-9\-_]+)[\s]/", $class, $sup ))
            $oclasses[$key]['sup'] = trim( $sup[1] );
        else
            $oclasses[$key]['sup'] = '';

        if(preg_match("/\s(STRUCTURAL|ABSTRACT|AUXILIARY)\s/",$class,$types))
            $oclasses[$key]['type'] = trim( $types[1] );
        else
            $oclasses[$key]['type'] ='';
   
        unset( $name );
        unset( $syntax );
        unset( $desription );
        unset( $types );

        // get all the required attributes
        if(preg_match( "/MUST[\s\(]+([a-zA-Z0-9\s$]+)(MAY|\))/" , $class, $must_attrs )){
            $must_attrs = str_replace( ' ', '', $must_attrs[1] );
            $oclasses[$key]['must_attrs'] = array_filter( explode( '$', $must_attrs ), "not_white" );
        }else{
            $must_attrs = '';
            $oclasses[$key]['must_attrs'] = array();
        }

        // get all the optional attributes
        if(preg_match( "/MAY[\s\(]+([a-zA-Z0-9\s$]+)(MUST|\))/" , $class, $may_attrs )){
            $may_attrs = str_replace( ' ', '', $may_attrs[1] );
            $oclasses[$key]['may_attrs'] = array_filter( array_merge( $oclasses[$key]['must_attrs'], explode( '$', $may_attrs) ), "not_white" );
        }else{
            $may_attrs = '';
            $oclasses[$key]['may_attrs'] = array();
        }
        unset( $must_attrs );
        unset( $may_attrs );

        $oclasses[$key]['name'] = $oclass_name;
    }

    // go back and add any inherited MUST/MAY attrs to each objectClass
    foreach( $oclasses as $oclass => $attrs )
    {
        $new_must = $attrs['must_attrs'];
        $new_may = $attrs['may_attrs'];
        $sup_attr = $attrs['sup'];

        while( $sup_attr && $sup_attr != "top" ) {
            $new_must = array_merge( $new_must, $oclasses[strtolower($sup_attr)]['must_attrs'] );
            $new_may = array_merge( $new_may, $oclasses[strtolower($sup_attr)]['may_attrs'] );
            $sup_attr = $oclasses[strtolower($sup_attr)]['sup'];
        }

        $oclasses[$oclass]['must_attrs'] = array_unique( $new_must );
        $oclasses[$oclass]['may_attrs'] = array_unique( $new_may );
    }

    ksort( $oclasses );

    return $oclasses;

}

/* 
 * Returns an associate array of the syntax OIDs that this LDAP server uses mapped to
 * their descriptions.
 */
function get_schema_syntaxes()
{
    $ds = conn();

    if( ! $ds )
        return false;

    // get all the attributeTypes
    $result = @ldap_read($ds, 'cn=subschema', '(objectClass=*)',
                array( 'ldapSyntaxes' ), 0, 200, 0, LDAP_DEREF_ALWAYS );
    if( ! $result )
    $result = @ldap_read($ds, 'cn=schema', '(objectClass=*)',
                array( 'ldapSyntaxes' ), 0, 200, 0, LDAP_DEREF_ALWAYS );

    if( $result )
        $raw = ldap_get_entries( $ds, $result );
    else
        return( array() );

    // build the array of attributes
    $syntaxes = array();
    for( $i=0; $i < $raw[0]['ldapsyntaxes']['count']; $i++ )
    {
        $syntax = $raw[0]['ldapsyntaxes'][$i];
        if(preg_match( "/[\s]+([\d\.]+)[\s]+/", $syntax, $oid)){
            $key = strtolower( trim( $oid[1] ) );
            if( ! $key ) continue;
            $syntaxes[$key] = array();
        }
        if(preg_match( "/[\s]+DESC[\s]+'([\)\(:?\.a-zA-Z0-9\-_ ]+)'/", $syntax, $description )){
            $syntaxes[$key]['description'] = $description[1];
        }
        if(preg_match("/X-NOT-HUMAN-READABLE\s+'TRUE'/i",$syntax) || preg_match("/X-BINARY-TRANSFER-REQUIRED\s+'TRUE'/i",$syntax)){
            $syntaxes[$key]['binary'] = 'Yes';
        }else{
            $syntaxes[$key]['binary'] = 'No';
        }

    }

    ksort( $syntaxes );

    return $syntaxes;
}

/* 
 * Returns an associative array of attributes for the specified 
 * $server_id. Each array entry's key is the name of the attribute,
 * in lower-case.
 * The sub-entries are 'oid', 'syntax', 'equality', 'substr', 'name',
 * and 'single_value'.
 *
 * The bulk of this function came from the good code in the 
 * GPL'ed LDAP Explorer project. Thank you. It was extended
 * considerably for application here.
 */
function get_schema_attributes()
{
    $ds = conn();

    if( ! $ds )
        return false;

    // get all the attributeTypes
    $result = @ldap_read($ds, 'cn=subschema', '(objectClass=*)',
                array( 'attributeTypes' ), 0, 200, 0, LDAP_DEREF_ALWAYS );
    if( ! $result )
        $result = @ldap_read($ds, 'cn=schema', '(objectClass=*)',
                array( 'attributeTypes' ), 0, 200, 0, LDAP_DEREF_ALWAYS );

    if( $result )
        $raw_attrs = ldap_get_entries( $ds, $result );
    else
        $raw_attrs = array();

    $syntaxes = get_schema_syntaxes();

    // build the array of attributes
    $attrs = array();
    for( $i=0; $i < $raw_attrs[0]['attributetypes']['count']; $i++ )
    {
        $attr = $raw_attrs[0]['attributetypes'][$i];

        #if(preg_match( "/[\s]+NAME[\s'\(]+([a-zA-Z0-9\-_]+)[\s'\)]/" , $attr, $name)){
        # ADDED code to deal with Multiple names
        $has_multiple_names = false;
        if(preg_match("/\sNAME\s+'([a-zA-Z0-9\-_]+)'\s+/",$attr,$name)){
            $key = strtolower( trim( $name[1] ) );
            $attr_name = trim( $name[1] );
            if( ! $key ) continue;
        }elseif(preg_match("/\sNAME\s+\(([a-zA-Z0-9\-_'\s]+)\)/",$attr,$name)){
            #multiple names
            $tmp_names = preg_replace("/^\s*'/",'',$name[1]);
            $tmp_names = preg_replace("/'\s*$/",'',$tmp_names);
            $names_arr = preg_split("/'\s*'/",$tmp_names,-1,PREG_SPLIT_NO_EMPTY);
            $attr_name = array_shift($names_arr);
            $attr_name = trim($attr_name);
            $key = strtolower($attr_name);
            if(empty($key)) continue;
            if(count($names_arr)>0){
                $has_multiple_names = true;
            }
        }
        $attrs[$key] = array();
        $attrs[$key]['name'] = $attr_name;

        if(preg_match( "/[\s]+([\d\.]+)[\s]+NAME/", $attr, $oid ))
            $attrs[$key]['oid'] = trim( $oid[1] );
        else
            $attrs[$key]['oid'] = '';
        
        if(preg_match( "/[\s]+DESC[\s]+'([\)\(:?\.a-zA-Z0-9\-_ ]+)'/", $attr, $description ))
            $attrs[$key]['description'] = trim( $description[1] );
        else
            $attrs[$key]['description'] = '';
        
        if(preg_match( "/[\s]+SYNTAX[\s]+([\d\.]+)/", $attr, $syntax)){
            $attrs[$key]['syntax'] = trim( $syntax[1] );
            @$attrs[$key]['type'] = $syntaxes[ trim($syntax[1]) ]['description'];
            @$attrs[$key]['binary'] = $syntaxes[ trim($syntax[1]) ]['binary'];
        }else{
            $attrs[$key]['syntax'] = '';
            $attrs[$key]['type'] = 'string';
            $attrs[$key]['binary'] = 'No';
        }
        
        if(preg_match( "/[\s]+EQUALITY[\s]+([a-zA-Z]+)/", $attr, $equality))
            $attrs[$key]['equality'] = trim( $equality[1] );
        else
            $attrs[$key]['equality'] = '';
        
        if(preg_match( "/[\s]+SUBSTR[\s]+([a-zA-Z]+)/", $attr, $substr))
            $attrs[$key]['substr'] = trim( $substr[1] );
        else
            $attrs[$key]['substr'] = '';

        if(preg_match( "/[\s]+SUP[\s]+([a-zA-Z0-9\-_]+)/", $attr, $sup ))
            $attrs[$key]['sup'] = trim( $sup[1] );
        else
            $attrs[$key]['sup'] = '';

        if( preg_match( "/[\s]+SINGLE-VALUE[\s]+/", $attr, $single_value ) )
            $attrs[$key]['single_value'] = 'Yes';
        else
            $attrs[$key]['single_value'] = 'No';

        # Now check if we need to populate other 'names' of his array
        if($has_multiple_names){
            $val = $attrs[$key];
            foreach($names_arr as $alias_tmp){
                $alias_name  = trim($alias_tmp);
                $alias = strtolower($alias_name);
                ##
                $attrs[$alias] = $val;
                $attrs[$alias]['name'] = $alias_name;
                ##
            }
        }

    }

    // go back and add any inherited descriptions from parent attributes (ie, cn inherits name)
    foreach( $attrs as $attr => $desc ){
    ###
        if(empty($desc['syntax']) && !empty($desc['sup']) ){
            $cattr = strtolower($desc['sup']);
            while(empty($attrs[$cattr]['syntax']) && !empty($attrs[$cattr]['sup']) ){
                $cattr = strtolower($attrs[$cattr]['sup']);
            }
            if(empty($desc['syntax'])){
                $attrs[$attr]['syntax'] = $attrs[$cattr]['syntax'];
            }
            if(empty($desc['type']) || $desc['type'] == 'string'){
                $attrs[$attr]['type'] = $attrs[$cattr]['type'];
            }
            if(empty($desc['equality'])){
                $attrs[$attr]['equality'] = $attrs[$cattr]['equality'];
            }
            if(empty($desc['substr'])){
                $attrs[$attr]['substr'] = $attrs[$cattr]['substr'];
            }
            if($desc['single_value'] == 'No'){
                $attrs[$attr]['single_value'] = $attrs[$cattr]['single_value'];
            }
            if($desc['binary'] == 'No'){
                $attrs[$attr]['binary'] = $attrs[$cattr]['binary'];
            }
        }
    ###
#        $sup_attr = $desc['sup'];
#        while( !empty($sup_attr) ) {
#            if( !empty($attrs[ $sup_attr ]['sup']) )  {
#                $attrs[ $attr ][ 'syntax' ]  = $attrs[ $sup_attr ]['syntax'];
#                $attrs[ $attr ][ 'equality' ]  = $attrs[ $sup_attr ]['equality'];
#                $attrs[ $attr ][ 'substr' ]  = $attrs[ $sup_attr ]['substr'];
#                $attrs[ $attr ][ 'single_value' ]  = $attrs[ $sup_attr ]['single_value'];
#                break;
#            } else {
#                @$sup_attr = $attrs[ $sup_attr ]['sup'];
#            }
#        }
    }

    ksort( $attrs );

    return $attrs;
}

/* 
 * A wrapper function to save you from having to call get_schema_objectclasses()
 * and get_schema_attributes(). Returns an array with two indexes: 'oclasses'
 * and 'attributes', as defined by their respective functions above.
 */
function get_schema()
{
    $ds = conn();

    if( ! $ds )
        return false;

    $attrs = get_schema_attributes();
    $oclasses = get_schema_objectclasses();

    if( ! $oclasses )
        return false;

    $schema = array( 'attrs' => $attrs,
             'oclasses' => $oclasses );
    ##
    ## Adding some Edits to the schema,
    ##
    @$schema['attrs']['photo']['type'] = 'Binary';
    @$schema['attrs']['preferreddeliverymethod']['type'] = 'Printable String';
    @$schema['attrs']['teletexterminalidentifier']['type'] = 'Directory String';
    return $schema;
}

?>
